/**
 * Copyright 1993 Network Computing Devices, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 *
 * THIS SOFTWARE IS PROVIDED `AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:	Greg Renda <greg@ncd.com>
 * 		Network Computing Devices, Inc.
 * 		350 North Bernardo Ave.
 * 		Mountain View, CA  94043
 *
 * $NCDId: @(#)audemo.c,v 1.36 1993/03/16 00:14:31 greg Exp $
 */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>
#include <audio/audiolib.h>
#include <audio/soundlib.h>

#define AddToLinkedList(head, item)					       \
{									       \
    (item)->prev = NULL;						       \
    (item)->next = head;						       \
    if (head)								       \
	(head)->prev = item;						       \
    head = item;							       \
}

#define RemoveFromLinkedList(head, item)				       \
{									       \
    if ((item)->next)							       \
	(item)->next->prev = (item)->prev;				       \
									       \
    if ((item)->prev)							       \
	(item)->prev->next = (item)->next;				       \
    else								       \
	head = (item)->next;						       \
}

#define XT

#ifdef XT
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include <audio/Xtutil.h>

/* widgets */
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Toggle.h>
#endif						/* XT */

#define APP_INSTANCE		"audemo"
#define	APP_CLASS		"Audemo"
#define INFO_COLS		50
#define INFO_ROWS		10
#define SAMPLES_ROWS		10
#define BUCKETS_COLS		strlen(BUCKET_HEADER)
#define BUCKETS_ROWS		5
#define	VOLUME_FORMAT		"Volume: %3d%%"
#define	GAIN_FORMAT		"Gain: %3d%%"
#define	DEFAULT_VOLUME		10
#define BUF_SIZE 		200
#define	MAX_VOLUME		200
#define	MIN_VOLUME		1
#define	MAX_GAIN		100
#define	MIN_GAIN		1
#define	DOUBLE_CLICK_TIME	500000		/* in microseconds */
#define DEFAULT_RECORD_FILE	"record.snd"
#define	DEFAULT_RECORD_RATE	"8000"
#define DEFAULT_RECORD_DURATION "10"
#define DEFAULT_TEMPLATE	"*.snd *.au *.wav *.voc"

#define VOL			AuFixedPointFromFraction(globals->volume, 100)
#define GAIN			AuFixedPointFromSum(globals->rec.gain, 0)

#define BUCKET_HEADER							       \
"    Description       Duration  Tracks  Frequency  Format  Access"
#define BUCKET_HEADER_FORMAT \
"%-20s  %5.2f     %2d     %5d Hz     %c      %s"


#define	HELP_INFO "\
Select a file from the list below.  Single clicking on the file will \
display information about the file in this window.  Double clicking on \
the file or clicking on the 'Play' button will play the file."

static String   defaultResources[] =
{
    "*font:		*courier-medium-r-normal*140*",
    NULL
};

static char   **makeFileList();
static int      getFileNames();
static void     fatalError(), bucketQueryCB(), bucketRecordDismissCB(),
                meterCB();

#ifdef XT
#define Invert(w)							       \
{									       \
    Pixel fg, bg;							       \
									       \
    XtVaGetValues(w, XtNforeground, &fg, XtNbackground, &bg, NULL);	       \
    XtVaSetValues(w, XtNforeground, bg, XtNbackground, fg, NULL);	       \
}

#define makeArg(resource, value)					       \
    XtSetArg(args[i], resource, (XtArgVal) (value)); i++

#define Anchor()							       \
    makeArg(XtNtop, XawChainTop);					       \
    makeArg(XtNbottom, XawChainTop);					       \
    makeArg(XtNleft, XawChainLeft);					       \
    makeArg(XtNright, XawChainLeft);					       \

#define MakeCommandButton(w, parent, label, horiz, vert, callback)	       \
{									       \
    Arg args[6];							       \
    int i = 0;								       \
									       \
    Anchor();								       \
    makeArg(XtNfromHoriz, horiz);					       \
    makeArg(XtNfromVert, vert);						       \
									       \
    (w) = XtCreateManagedWidget(label, commandWidgetClass, parent, args, i);   \
									       \
    if (callback)							       \
	XtAddCallback(w, XtNcallback, callback, globals);		       \
}

#define MakeLabel(w, parent, label, horiz, vert)			       \
{									       \
    Arg args[7];							       \
    int i = 0;								       \
									       \
    Anchor();								       \
    makeArg(XtNfromHoriz, horiz);					       \
    makeArg(XtNfromVert, vert);						       \
    makeArg(XtNborderWidth, 0);						       \
									       \
    (w) = XtCreateManagedWidget(label, labelWidgetClass, parent, args, i);     \
}

typedef struct
{
    Widget          popShell,
                    form,
                    record,
                    dismiss,
                    file,
                    rate,
                    monitor,
                    menu,
                    menuButton,
                    comment,
                    duration,
                    new,
                    readOnly,
                    mode,
                    gainLabel,
                    gainBar;
    unsigned int    gain;
}               RecordDialogDataRec, *RecordDialogDataPtr;

typedef struct
{
    Widget          popShell,
                    form,
                    dismiss,
                    view,
                    bucketList,
                    query,
                    play,
                    record,
                    load,
                    delete,
                    save,
                    dialogShell,
                    formatMenu,
                    formatMenuButton,
                    accessMenu,
                    accessMenuButton,
                    dialog;
    int             numBuckets;
    AuBucketAttributes *list;
    char          **bucketText;
}               BucketDialogDataRec, *BucketDialogDataPtr;

typedef struct
{
    Widget          topLevel,
                    meter,
                    quit,
                    form,
                    play,
                    samples,
                    info,
                    view,
                    volumeBar,
                    volumeLabel,
                    leftMeter,
                    rightMeter,
                    record,
                    buckets,
                    dir,
                    template,
                    rescan;
    int             volume,
                    numFiles,
                    sourceIndex;
    AuServer       *aud;
    char          **fileNames;
    Display        *dpy;
    RecordDialogDataRec rec;
    BucketDialogDataRec buf;
    AuDeviceID      inputDeviceId;
}               GlobalDataRec, *GlobalDataPtr;

typedef struct _ElementList
{
    AuFlowID        flow;
    int             volumeElement,
                    monitorElement;
    struct _ElementList *prev,
                   *next;
}               ElementListRec, *ElementListPtr;

typedef ElementListPtr ElementListId;

static ElementListPtr ElementList;
static int      ElementCount,
                MonitorCount;

#ifndef sun					/* who else doesn't have
						 * this? */
char           *
strdup(s)
char           *s;
{
    char           *t;

    if (!s)
	return s;
    t = (char *) malloc(strlen(s) + 1);
    strcpy(t, s);
    return t;
}
#endif

static void
RemoveFromElementList(globals, p)
GlobalDataPtr   globals;
ElementListPtr  p;
{
    RemoveFromLinkedList(ElementList, p);
    ElementCount--;

    if (p->monitorElement != -1)
    {
	MonitorCount--;

	if (!MonitorCount)
	    meterCB(globals->aud, NULL, NULL, globals);
    }

    free(p);
}

static          ElementListId
AddToElementList(flow, volumeElement, monitorElement)
AuFlowID        flow;
int             volumeElement,
                monitorElement;
{
    ElementListPtr  p;

    if (!(p = (ElementListPtr) malloc(sizeof(ElementListRec))))
	fatalError("malloc error in AddToElementList");

    p->flow = flow;
    p->volumeElement = volumeElement;
    p->monitorElement = monitorElement;

    AddToLinkedList(ElementList, p);
    ElementCount++;

    if (monitorElement != -1)
	MonitorCount++;

    return (ElementListId) p;
}

static void
queryInputAttributes(globals)
GlobalDataPtr   globals;
{
    Boolean         mode;
    AuDeviceAttributes *attr;
    char            buf[50];

    attr = AuGetDeviceAttributes(globals->aud, globals->inputDeviceId, NULL);
    globals->rec.gain = AuFixedPointRoundDown(AuDeviceGain(attr));
    sprintf(buf, GAIN_FORMAT, globals->rec.gain);
    XtVaSetValues(globals->rec.gainLabel, XtNlabel, buf, NULL);
    XawScrollbarSetThumb(globals->rec.gainBar,
			 (float) globals->rec.gain / MAX_GAIN, -1.0);

    mode = AuDeviceLineMode(attr) == AuDeviceLineModeHigh;
    XtVaSetValues(globals->rec.mode, XtNstate, mode, NULL);

    AuFreeDeviceAttributes(globals->aud, 1, attr);
}

static void
meterToggleCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    Boolean         meter;
    AuElementState *states,
                   *s;
    ElementListPtr  p;
    int             state;

    XtVaGetValues(globals->meter, XtNstate, &meter, NULL);

    state = meter ? AuStateStart : AuStateStop;

    if (MonitorCount)
    {
	if (!(states = (AuElementState *) malloc(sizeof(AuElementState) *
						 MonitorCount)))
	    return;

	p = ElementList;
	s = states;

	while (p)
	{
	    if (p->monitorElement != -1)
	    {
		AuMakeElementState(s, p->flow, p->monitorElement, state);
		s++;
	    }

	    p = p->next;
	}

	AuSetElementStates(globals->aud, MonitorCount, states, NULL);
	free(states);
	meterCB(globals->aud, NULL, NULL, globals);
    }
}

#define MonitorNotifyEventMin(e, i) ((short *) &(e)->data)[i * (e)->num_fields]
#define MonitorNotifyEventMax(e, i)					       \
    ((short *) &(e)->data)[i * (e)->num_fields + 1]

static void
meterCB(aud, handler, ev, globals)
AuServer       *aud;
AuEventHandlerRec *handler;
AuEvent        *ev;
GlobalDataPtr   globals;
{
    AuMonitorNotifyEvent *e = (AuMonitorNotifyEvent *) ev;
    static int      count;
    static float    left,
                    right;
    float           l;
    Boolean         meter;


    if (!ev)
    {
	XawScrollbarSetThumb(globals->leftMeter, -1.0, 0);
	XawScrollbarSetThumb(globals->rightMeter, -1.0, 0);
	count = left = right = 0;
	return;
    }

    XtVaGetValues(globals->meter, XtNstate, &meter, NULL);

    if (!meter)
	return;

    l = (float) (MonitorNotifyEventMax(e, 0) -
		 MonitorNotifyEventMin(e, 0)) / 65535;

    left += l;

    if (e->num_tracks == 1)
	right += l;
    else
	right += (float) (MonitorNotifyEventMax(e, 1) -
			  MonitorNotifyEventMin(e, 1)) / 65535;

    count++;

    if (count >= MonitorCount)
    {
	left /= MonitorCount;
	right /= MonitorCount;

	XawScrollbarSetThumb(globals->leftMeter, -1.0, left);
	XawScrollbarSetThumb(globals->rightMeter, -1.0, right);
	count = left = right = 0;
    }
}

typedef struct
{
    GlobalDataPtr   globals;
    void            (*callback) ();
}               DonePrivRec, *DonePrivPtr;

static void
doneCB(aud, handler, ev, data)
AuServer       *aud;
AuEvent        *ev;
AuEventHandlerRec *handler;
DonePrivPtr     data;
{
    if (ev->type == AuEventTypeMonitorNotify)
	meterCB(aud, handler, ev, data->globals);
    else
	(*data->callback) (NULL, data->globals, data);
}

static void
modeCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    Boolean         mode;
    AuDeviceAttributes a;

    XtVaGetValues(globals->rec.mode, XtNstate, &mode, NULL);
    AuDeviceValueMask(&a) = AuCompDeviceLineModeMask;
    AuDeviceLineMode(&a) = mode ? AuDeviceLineModeHigh : AuDeviceLineModeLow;
    AuSetDeviceAttributes(globals->aud, globals->inputDeviceId,
			  AuCompDeviceLineModeMask, &a, NULL);
}

static void
newBucketCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    String          s;
    unsigned int    rate,
                    maxDuration,
                    format;
    Boolean         readOnly;
    AuString        desc;

    XtVaGetValues(globals->rec.rate, XtNstring, &s, NULL);
    rate = atoi(s);
    XtVaGetValues(globals->rec.duration, XtNstring, &s, NULL);
    maxDuration = atoi(s);
    XtVaGetValues(globals->rec.readOnly, XtNstate, &readOnly, NULL);
    XtVaGetValues(globals->rec.menuButton, XtNlabel, &s, NULL);
    format = AuStringToFormat(s);
    XtVaGetValues(globals->rec.comment, XtNstring, &s, NULL);

    desc.type = AuStringLatin1;
    desc.len = strlen(s);
    desc.data = s;

    AuCreateBucket(globals->aud, format, 1,
		   readOnly ? AuAccessImportMask | AuAccessListMask :
		   AuAccessAllMasks, rate, maxDuration * rate, &desc, NULL);

    bucketQueryCB(globals->buf.query, globals, NULL);
}

static void
rescanCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    static char    *noFilesString = "No files found";
    char           *dir,
                   *template;
    int             i;

    if (globals->numFiles)
    {
	for (i = 0; i < globals->numFiles; i++)
	    free(globals->fileNames[i]);

	free(globals->fileNames);
    }

    XtVaGetValues(globals->dir, XtNstring, &dir, NULL);
    XtVaGetValues(globals->template, XtNstring, &template, NULL);
    globals->numFiles = getFileNames(dir, &globals->fileNames, template);

    if (globals->numFiles)
    {
	XawListChange(globals->samples,
		      makeFileList(globals->fileNames, globals->numFiles),
		      globals->numFiles, -1, True);
	XtSetSensitive(globals->samples, True);
    }
    else
    {
	XawListChange(globals->samples, &noFilesString, 1, -1, True);
	XtSetSensitive(globals->samples, False);
    }
}

static void
bucketPlayCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    static Boolean  playing;
    static AuFlowID flow;
    static int      multiplier;
    static DonePrivRec priv;
    static ElementListId listid;
    XawListReturnStruct *sel;
    BucketDialogDataPtr buf = &globals->buf;
    Boolean         meter;
    int             index,
                    monitor = -1;

    if (playing)
    {
	if (w)
	    /* user requested stop */
	    AuStopFlow(globals->aud, flow, NULL);
	else
	{
	    /* got a done callback */
	    RemoveFromElementList(globals, listid);
	    playing = False;
	    Invert(globals->buf.play);
	}

	return;
    }
    else if (w != globals->buf.play)
	return;

    sel = XawListShowCurrent(buf->bucketList);

    if ((index = sel->list_index) == -1)
	return;

    priv.globals = globals;
    priv.callback = bucketPlayCB;

    XtVaGetValues(globals->meter, XtNstate, &meter, NULL);

    if (AuSoundPlayFromBucket(globals->aud,
			      AuBucketIdentifier(&buf->list[index]), AuNone,
			      VOL, doneCB, &priv, 1, &flow, &multiplier,
			      meter ? &monitor : NULL, NULL))
    {
	listid = AddToElementList(flow, multiplier, monitor);
	playing = True;
	Invert(globals->buf.play);
    }
}

#define COMMENT_LEN 20

static void
bucketQueryCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    BucketDialogDataPtr buf = &globals->buf;
    char            tmp[200],
                    access[4],
                    desc[COMMENT_LEN];
    int             i;
    static char    *noBucketString = "No buckets";

    if (buf->numBuckets)
    {
	AuFreeBucketAttributes(globals->aud, buf->numBuckets, buf->list);

	for (i = 0; i < buf->numBuckets; i++)
	    free(buf->bucketText[i]);

	free(buf->bucketText);
    }

    buf->list = AuListBuckets(globals->aud, 0, NULL, &buf->numBuckets, NULL);

    if (!(buf->bucketText = (char **) malloc(sizeof(char *) * buf->numBuckets)))
	fatalError("malloc error in bucketQueryCB");

    for (i = 0; i < buf->numBuckets; i++)
    {
	AuBucketAttributes *p;
	int             j;
	char            format;

	p = &buf->list[i];
	*desc = 0;
	*access = 0;

	if (AuBucketValueMask(p) & AuCompCommonDescriptionMask)
	{
	    strncpy(desc, AuBucketDescription(p)->data, COMMENT_LEN - 1);
	    desc[COMMENT_LEN - 1] = 0;
	}

	if (AuBucketAccess(p) & AuAccessImportMask)
	    strcat(access, "I");
	if (AuBucketAccess(p) & AuAccessExportMask)
	    strcat(access, "E");
	if (AuBucketAccess(p) & AuAccessDestroyMask)
	    strcat(access, "D");

	for (j = 0, format = 'A'; j < AuServerNumFormats(globals->aud);
	     j++, format++)
	    if (AuBucketFormat(p) == AuServerFormat(globals->aud, j))
		break;

	sprintf(tmp, BUCKET_HEADER_FORMAT,
		desc, (float) AuBucketNumSamples(p) / AuBucketSampleRate(p),
		AuBucketNumTracks(p), AuBucketSampleRate(p), format, access);

	buf->bucketText[i] = strdup(tmp);
    }

    if (buf->numBuckets)
    {
	XawListChange(buf->bucketList, buf->bucketText, buf->numBuckets, -1,
		      True);
	XtSetSensitive(buf->bucketList, True);
    }
    else
    {
	XawListChange(buf->bucketList, &noBucketString, 1, -1, True);
	XtSetSensitive(buf->bucketList, False);
    }
}

static void
bucketDeleteCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    XawListReturnStruct *sel;
    BucketDialogDataPtr buf = &globals->buf;
    int             index;

    sel = XawListShowCurrent(buf->bucketList);

    if ((index = sel->list_index) == -1)
	return;

    AuDestroyBucket(globals->aud, AuBucketIdentifier(&buf->list[index]), NULL);
    bucketQueryCB(buf->query, globals, NULL);
}

static void
bucketLoadCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    XawListReturnStruct *sel;
    int             index;

    sel = XawListShowCurrent(globals->samples);

    if ((index = sel->list_index) == -1)
	return;

    AuSoundCreateBucketFromFile(globals->aud, globals->fileNames[index],
				AuAccessAllMasks, NULL, NULL);
    bucketQueryCB(globals->buf.query, globals, NULL);
}

static void
bucketsCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    bucketQueryCB(globals->buf.query, globals, 0);
    XtSetSensitive(w, False);
    XtPopup(globals->buf.popShell, XtGrabNone);
}

static void
bucketRecordStartCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    static Boolean  recording;
    static DonePrivRec priv;
    static AuFlowID flow;
    Boolean         mode;
    XawListReturnStruct *sel;
    BucketDialogDataPtr buf = &globals->buf;
    int             index;

    if (recording)
    {
	if (w)
	    /* user requested stop */
	    AuStopFlow(globals->aud, flow, NULL);
	else
	{
	    /* got a done callback */
	    recording = False;
	    Invert(globals->rec.record);
	}

	return;
    }
    else if (w != globals->rec.record)
	return;

    sel = XawListShowCurrent(buf->bucketList);

    if ((index = sel->list_index) == -1)
	return;

    priv.globals = globals;
    priv.callback = bucketRecordStartCB;

    XtVaGetValues(globals->rec.mode, XtNstate, &mode, NULL);

    if (AuSoundRecordToBucket(globals->aud,
			      AuBucketIdentifier(&buf->list[index]),
			      globals->inputDeviceId, GAIN, doneCB, &priv,
			  mode ? AuDeviceLineModeHigh : AuDeviceLineModeLow,
			      &flow, NULL, NULL))
    {
	Invert(globals->rec.record);
	recording = True;
    }
}

static void
recordStartCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    static Boolean  recording;
    static DonePrivRec priv;
    static AuFlowID flow;
    String          s,
                    comment;
    char           *filename;
    unsigned long   rate,
                    format;
    Boolean         mode;

    if (recording)
    {
	if (w)
	    /* user requested stop */
	    AuStopFlow(globals->aud, flow, NULL);
	else
	{
	    /* got a done callback */
	    recording = False;
	    Invert(globals->rec.record);
	}

	return;
    }
    else if (w != globals->rec.record)
	return;

    priv.globals = globals;
    priv.callback = recordStartCB;

    XtVaGetValues(globals->rec.menuButton, XtNlabel, &s, NULL);
    format = AuStringToFormat(s);

    XtVaGetValues(globals->rec.rate, XtNstring, &s, NULL);
    rate = atoi(s);

    XtVaGetValues(globals->rec.mode, XtNstate, &mode, NULL);
    XtVaGetValues(globals->rec.comment, XtNstring, &comment, NULL);
    XtVaGetValues(globals->rec.file, XtNstring, &s, NULL);

    if (AuSoundRecordToFile(globals->aud, s, globals->inputDeviceId, GAIN,
			    doneCB, &priv,
			  mode ? AuDeviceLineModeHigh : AuDeviceLineModeLow,
			    SND_MAGIC, comment, rate, format,
			    &flow, NULL, NULL))
    {
	Invert(globals->rec.record);
	recording = True;
    }
}

static void
monitorCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    static Boolean  monitoring;
    static AuFlowID flow;
    static int      multiplier;
    static DonePrivRec priv;
    static ElementListId listid;
    String          s;
    Boolean         meter;
    int             rate,
                    monitor = -1;

    if (monitoring)
    {
	if (w)
	    /* user requested stop */
	    AuStopFlow(globals->aud, flow, NULL);
	else
	{
	    /* got a done callback */
	    RemoveFromElementList(globals, listid);
	    monitoring = False;
	    Invert(globals->rec.monitor);
	}

	return;
    }
    else if (w != globals->rec.monitor)
	return;

    XtVaGetValues(globals->rec.rate, XtNstring, &s, NULL);
    rate = atoi(s);

    priv.globals = globals;
    priv.callback = monitorCB;

    XtVaGetValues(globals->meter, XtNstate, &meter, NULL);

    if (AuMonitorDevice(globals->aud, rate, globals->inputDeviceId, AuNone, VOL,
			doneCB, &priv, &flow, &multiplier,
			meter ? &monitor : NULL, NULL))
    {
	listid = AddToElementList(flow, multiplier, monitor);
	monitoring = True;
	Invert(globals->rec.monitor);
    }
}

static void
dialogCancelCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    XtPopdown(globals->buf.dialogShell);
    XtSetSensitive(globals->buf.save, True);
}

static void
dialogOkCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    String          filename;
    XawListReturnStruct *sel;
    BucketDialogDataPtr buf = &globals->buf;
    int             index;

    dialogCancelCB(w, globals, call_data);

    sel = XawListShowCurrent(buf->bucketList);

    if ((index = sel->list_index) == -1)
	return;

    filename = XawDialogGetValueString(buf->dialog);

    if (strlen(filename))
    {
	char           *p;

	if (!(p = (char *) malloc(strlen(filename) + 5)))
	    fatalError("malloc error in dialogOkCB");

	sprintf(p, "%s.snd", filename);
	AuSoundCreateFileFromBucket(globals->aud, p, SND_MAGIC,
				    AuBucketIdentifier(&buf->list[index]),
				    NULL);
	free(p);
    }

    rescanCB(globals->rescan, globals, 0);
}

static void
okAction(w, event, params, num_params)
Widget          w;
XEvent         *event;
String         *params;
Cardinal       *num_params;
{
    GlobalDataPtr   globals;

    globals = (GlobalDataPtr) atoi(params[0]);
    dialogOkCB(w, globals, 0);
}

static void
bucketSaveCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    int             index;
    XawListReturnStruct *sel;

    sel = XawListShowCurrent(globals->buf.bucketList);

    if (sel->list_index == -1)
	return;

    XtSetSensitive(w, False);
    XtPopup(globals->buf.dialogShell, XtGrabNonexclusive);
}

static void
bucketDismissCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    bucketRecordDismissCB(globals->rec.dismiss, globals, NULL);
    bucketPlayCB(NULL, globals, NULL);

    XtPopdown(globals->buf.popShell);
    XtSetSensitive(globals->buckets, True);
}

static void
recordDismissCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    monitorCB(w, globals, NULL);
    recordStartCB(w, globals, NULL);

    XtPopdown(globals->rec.popShell);
    XtSetSensitive(globals->record, True);
    XtSetSensitive(globals->buf.record, True);
}

static void
bucketRecordDismissCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    monitorCB(w, globals, NULL);
    bucketRecordStartCB(w, globals, NULL);

    XtPopdown(globals->rec.popShell);
    XtSetSensitive(globals->record, True);
    XtSetSensitive(globals->buf.record, True);
}

#define ReplaceCallback(w, cb)						       \
{									       \
    XtRemoveAllCallbacks(w, XtNcallback);				       \
    XtAddCallback(w, XtNcallback, cb, globals);				       \
}

static void
recordSensitives(globals, state)
GlobalDataPtr   globals;
Boolean         state;
{
    XtSetSensitive(globals->rec.file, state);
}

static void
bucketRecordSensitives(globals, state)
GlobalDataPtr   globals;
Boolean         state;
{
    XtSetSensitive(globals->rec.duration, state);
    XtSetSensitive(globals->rec.readOnly, state);
    XtSetSensitive(globals->rec.new, state);
}

static void
recordCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    ReplaceCallback(globals->rec.record, recordStartCB);
    ReplaceCallback(globals->rec.dismiss, recordDismissCB);

    recordSensitives(globals, True);
    bucketRecordSensitives(globals, False);
    XtSetSensitive(globals->buf.record, False);

    queryInputAttributes(globals);

    XtSetSensitive(w, False);
    XtPopup(globals->rec.popShell, XtGrabNone);
}

static void
bucketRecordCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    ReplaceCallback(globals->rec.record, bucketRecordStartCB);
    ReplaceCallback(globals->rec.dismiss, bucketRecordDismissCB);

    recordSensitives(globals, False);
    bucketRecordSensitives(globals, True);
    XtSetSensitive(globals->record, False);

    queryInputAttributes(globals);

    XtSetSensitive(w, False);
    XtPopup(globals->rec.popShell, XtGrabNone);
}

static void
playCB(w, globals, data)
Widget          w;
GlobalDataPtr   globals;
DonePrivPtr     data;
{
    static Boolean  playing;
    static AuFlowID flow;
    static int      multiplier;
    static DonePrivRec priv;
    static ElementListId listid;
    XawListReturnStruct *sel;
    Boolean         meter;
    int             index,
                    monitor = -1;

    if (playing)
    {
	if (w)
	    /* user requested stop */
	    AuStopFlow(globals->aud, flow, NULL);
	else
	{
	    /* got a done callback */
	    RemoveFromElementList(globals, listid);
	    playing = False;
	    Invert(globals->play);
	}

	return;
    }

    sel = XawListShowCurrent(globals->samples);

    if ((index = sel->list_index) == -1)
	return;

    priv.globals = globals;
    priv.callback = playCB;

    XtVaGetValues(globals->meter, XtNstate, &meter, NULL);

    if (AuSoundPlayFromFile(globals->aud, globals->fileNames[index], AuNone,
			    VOL, doneCB, &priv, &flow, &multiplier,
			    meter ? &monitor : NULL, NULL))
    {
	listid = AddToElementList(flow, multiplier, monitor);
	playing = True;
	Invert(globals->play);
    }
}

static void
bucketListCB(w, globals, listInfo)
Widget          w;
GlobalDataPtr   globals;
XawListReturnStruct *listInfo;
{
    struct timeval  tp;
    unsigned long   currentTime;
    static long     lastTime;
    static int      lastSelection;

    gettimeofday(&tp, NULL);
    currentTime = tp.tv_sec * 1000000 + tp.tv_usec;

    if (listInfo->list_index == lastSelection &&
	currentTime - lastTime < DOUBLE_CLICK_TIME)
	bucketPlayCB(globals->buf.play, globals, NULL);

    lastTime = currentTime;
    lastSelection = listInfo->list_index;
}

static void
samplesCB(w, globals, listInfo)
Widget          w;
GlobalDataPtr   globals;
XawListReturnStruct *listInfo;
{
    struct timeval  tp;
    long            currentTime;
    static long     lastTime;
    static int      lastSelection = -1;

    gettimeofday(&tp, NULL);
    currentTime = tp.tv_sec * 1000000 + tp.tv_usec;

    if (listInfo->list_index == lastSelection &&
	currentTime - lastTime < DOUBLE_CLICK_TIME)
	playCB(globals->play, globals, (XtPointer) 0);
    else if (lastSelection != listInfo->list_index)
    {
	SoundHeader     hdr;
	FILE           *fp;
	char           *comment;

	lastSelection = listInfo->list_index;

	if (fp = SoundOpenFileForReading(globals->fileNames[lastSelection],
					 &hdr, &comment))
	{
	    char           *buf,
	                   *p;
	    int             dummy;

#define PRINT(p, f, a)							       \
{									       \
    sprintf(p, f, a);							       \
    p += strlen(p);							       \
}

	    if (!(p = buf = (char *) malloc(2000 + strlen(comment))))
		fatalError("Can't malloc text in samplesCB");

	    PRINT(p, " Filename: %s\n", globals->fileNames[lastSelection]);
	    PRINT(p, "   Format: %s\n", AuFormatToString(hdr.format));
	    PRINT(p, "   Tracks: %d\n", hdr.tracks);
	    PRINT(p, "Frequency: %d Hz\n", hdr.sampleRate);
	    PRINT(p, " Duration: %.2f seconds\n", (float) hdr.dataSize /
		  hdr.tracks / hdr.sampleRate /
		  SoundSizeofFormat(hdr.format));

	    PRINT(p, "\n%s", comment);

	    XtVaSetValues(globals->info, XtNstring, buf, NULL);

	    free(buf);
	    free(comment);
	    fclose(fp);
	}
    }

    gettimeofday(&tp, NULL);
    lastTime = tp.tv_sec * 1000000 + tp.tv_usec;
}

static void
quitCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    exit(0);
}

static void
adjustVolume(globals)
GlobalDataPtr   globals;
{
    AuElementParameters *parms;
    ElementListPtr  p = ElementList;
    int             i = 0;

    if (!ElementCount)
	return;

    if (!(parms = (AuElementParameters *)
	  malloc(sizeof(AuElementParameters) * ElementCount)))
	fatalError("malloc error in adjustVolume");

    while (p)
    {
	parms[i].flow = p->flow;
	parms[i].element_num = p->volumeElement;
	parms[i].num_parameters = AuParmsMultiplyConstant;
	parms[i].parameters[AuParmsMultiplyConstantConstant] = VOL;

	p = p->next;
	i++;
    }

    AuSetElementParameters(globals->aud, ElementCount, parms, NULL);
    free(parms);
}

static void
scrollProcCB(w, globals, position)
Widget          w;
GlobalDataPtr   globals;
int             position;
{
    int             newVolume;
    char            buf[50];

    newVolume = globals->volume + (position > 0 ? -1 : 1);

    if (newVolume < MIN_VOLUME)
	newVolume = MIN_VOLUME;
    else if (newVolume > MAX_VOLUME)
	newVolume = MAX_VOLUME;

    if (newVolume != globals->volume)
    {
	globals->volume = newVolume;
	sprintf(buf, VOLUME_FORMAT, globals->volume);
	XtVaSetValues(globals->volumeLabel, XtNlabel, buf, NULL);

	XawScrollbarSetThumb(globals->volumeBar,
			     (float) globals->volume / MAX_VOLUME, -1.0);
	adjustVolume(globals);
    }
}

static void
jumpProcCB(w, globals, percent)
Widget          w;
GlobalDataPtr   globals;
float          *percent;
{
    Arg             args[1];
    int             newVolume;
    char            buf[50];

    newVolume = *percent * MAX_VOLUME;

    if (newVolume < MIN_VOLUME)
	newVolume = MIN_VOLUME;

    if (newVolume != globals->volume)
    {
	globals->volume = newVolume;
	sprintf(buf, VOLUME_FORMAT, globals->volume);
	XtVaSetValues(globals->volumeLabel, XtNlabel, buf, NULL);
	adjustVolume(globals);
    }
}

static void
adjustGain(globals)
GlobalDataPtr   globals;
{
    AuDeviceAttributes a;

    AuDeviceValueMask(&a) = AuCompDeviceGainMask;
    AuDeviceGain(&a) = GAIN;
    AuSetDeviceAttributes(globals->aud, globals->inputDeviceId,
			  AuCompDeviceGainMask, &a, NULL);
}

static void
gainScrollCB(w, globals, position)
Widget          w;
GlobalDataPtr   globals;
int             position;
{
    int             newGain;
    char            buf[50];

    newGain = globals->rec.gain + (position > 0 ? -1 : 1);

    if (newGain < MIN_GAIN)
	newGain = MIN_GAIN;
    else if (newGain > MAX_GAIN)
	newGain = MAX_GAIN;

    if (newGain != globals->rec.gain)
    {
	globals->rec.gain = newGain;
	sprintf(buf, GAIN_FORMAT, globals->rec.gain);
	XtVaSetValues(globals->rec.gainLabel, XtNlabel, buf, NULL);

	XawScrollbarSetThumb(globals->rec.gainBar,
			     (float) globals->rec.gain / MAX_GAIN, -1.0);
	adjustGain(globals);
    }
}

static void
gainJumpCB(w, globals, percent)
Widget          w;
GlobalDataPtr   globals;
float          *percent;
{
    Arg             args[1];
    int             newGain;
    char            buf[50];

    newGain = *percent * MAX_GAIN;

    if (newGain < MIN_GAIN)
	newGain = MIN_GAIN;

    if (newGain != globals->rec.gain)
    {
	globals->rec.gain = newGain;
	sprintf(buf, GAIN_FORMAT, globals->rec.gain);
	XtVaSetValues(globals->rec.gainLabel, XtNlabel, buf, NULL);
	adjustGain(globals);
    }
}

static void
menuCB(w, globals, call_data)
Widget          w;
GlobalDataPtr   globals;
XtPointer       call_data;
{
    String          string;

    XtVaGetValues(w, XtNlabel, &string, NULL);
    XtVaSetValues(globals->rec.menuButton, XtNlabel, string, NULL);
}

static          Dimension
getFontCharWidth(w)
Widget          w;
{
    XFontStruct    *font;

    XtVaGetValues(w, XtNfont, &font, NULL);
    return font->max_bounds.rbearing - font->min_bounds.lbearing;
}

static          Dimension
getFontCharHeight(w)
Widget          w;
{
    XFontStruct    *font;

    XtVaGetValues(w, XtNfont, &font, NULL);
    return font->max_bounds.ascent + font->max_bounds.descent;
}

#define TRANS "<Key>Return: beginning-of-line()"

static void
makeBucketDialog(globals)
GlobalDataPtr   globals;
{
    Arg             args[20];
    Widget          w;
    int             i;
    BucketDialogDataPtr buf = &globals->buf;
    char            tmp[100];
    static char     popupTransTable[] = "<Key>Return: Ok";

    buf->numBuckets = 0;
    buf->popShell = XtCreatePopupShell("Buckets", transientShellWidgetClass,
				       globals->topLevel, NULL, 0);

    buf->form = XtCreateManagedWidget("form", formWidgetClass, buf->popShell,
				      NULL, 0);

    MakeCommandButton(buf->query, buf->form, "Query", NULL, NULL,
		      bucketQueryCB);
    MakeCommandButton(buf->play, buf->form, "Play", buf->query, NULL,
		      bucketPlayCB);
    MakeCommandButton(buf->record, buf->form, "Record", buf->play, NULL,
		      bucketRecordCB);
    MakeCommandButton(buf->load, buf->form, "Load", buf->record, NULL,
		      bucketLoadCB);
    MakeCommandButton(buf->save, buf->form, "Save", buf->load, NULL,
		      bucketSaveCB);
    MakeCommandButton(buf->delete, buf->form, "Delete", buf->save, NULL,
		      bucketDeleteCB);
    MakeCommandButton(buf->dismiss, buf->form, "Dismiss", buf->delete, NULL,
		      bucketDismissCB);

    buf->formatMenu = XtCreatePopupShell("format", simpleMenuWidgetClass,
					 buf->form, NULL, 0);

    for (i = 0; i < AuServerNumFormats(globals->aud); i++)
    {
	sprintf(tmp, "%c - %s", 'A' + i,
		AuFormatToString(AuServerFormat(globals->aud, i)));
	XtCreateManagedWidget(tmp, smeBSBObjectClass, buf->formatMenu, NULL, 0);
    }

    i = 0;
    Anchor();
    makeArg(XtNfromHoriz, buf->dismiss);
    makeArg(XtNhorizDistance, 30);
    makeArg(XtNmenuName, "format");
    buf->formatMenuButton = XtCreateManagedWidget("Format Key",
						  menuButtonWidgetClass,
						  buf->form, args, i);

    buf->accessMenu = XtCreatePopupShell("access", simpleMenuWidgetClass,
					 buf->form, NULL, 0);

    XtCreateManagedWidget("I - Importable",
			  smeBSBObjectClass, buf->accessMenu, NULL, 0);
    XtCreateManagedWidget("E - Exportable",
			  smeBSBObjectClass, buf->accessMenu, NULL, 0);
    XtCreateManagedWidget("D - Destroyable",
			  smeBSBObjectClass, buf->accessMenu, NULL, 0);

    i = 0;
    Anchor();
    makeArg(XtNfromHoriz, buf->formatMenuButton);
    makeArg(XtNmenuName, "access");
    buf->accessMenuButton = XtCreateManagedWidget("Access Key",
						  menuButtonWidgetClass,
						  buf->form, args, i);

    MakeLabel(w, buf->form, BUCKET_HEADER, NULL, buf->dismiss);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNfromVert, w);
    makeArg(XtNallowVert, True);
    makeArg(XtNforceBars, True);
    makeArg(XtNwidth, getFontCharWidth(buf->query) * BUCKETS_COLS);
    makeArg(XtNheight, getFontCharHeight(buf->query) * BUCKETS_ROWS);
    buf->view = XtCreateManagedWidget("view", viewportWidgetClass,
				      buf->form, args, i);

    i = 0;
    makeArg(XtNallowVert, True);
    makeArg(XtNdefaultColumns, 1);
    makeArg(XtNforceColumns, True);
    makeArg(XtNresizable, True);
    buf->bucketList = XtCreateManagedWidget("bucketList", listWidgetClass,
					    buf->view, args, i);
    XtAddCallback(buf->bucketList, XtNcallback, bucketListCB, globals);

    buf->dialogShell = XtCreatePopupShell("Save bucket",
					  transientShellWidgetClass,
					  buf->popShell, NULL, 0);

    i = 0;
    makeArg(XtNlabel, "Filename:");
    makeArg(XtNvalue, "");
    buf->dialog = XtCreateManagedWidget("dialog", dialogWidgetClass,
					buf->dialogShell, args, i);
    XawDialogAddButton(buf->dialog, "Ok", dialogOkCB, globals);
    XawDialogAddButton(buf->dialog, "Cancel", dialogCancelCB, globals);

    /* pass the globals as an argument to the callback */
    strcpy(tmp, popupTransTable);
    sprintf(tmp + strlen(tmp), "(%u)", (unsigned int) globals);

    XtOverrideTranslations(XtNameToWidget(buf->dialog, "value"),
			   XtParseTranslationTable(tmp));
}

static void
makeRecordDialog(globals)
GlobalDataPtr   globals;
{
    Arg             args[20];
    Widget          w;
    int             i,
                    j;
    XtTranslations  translations;
    RecordDialogDataPtr rec = &globals->rec;
    Dimension       width,
                    height;
    Position        x,
                    rightX;
    char            buf[50];
    String          s;
    static XtCallbackRec menuCallbacks[] =
    {
	menuCB, (caddr_t) NULL,
	(XtCallbackProc) NULL, (caddr_t) NULL
    };

    menuCallbacks[0].closure = (caddr_t) globals;
    translations = XtParseTranslationTable(TRANS);

    rec->popShell = XtCreatePopupShell("Record", transientShellWidgetClass,
				       globals->topLevel, NULL, 0);

    rec->form = XtCreateManagedWidget("form", formWidgetClass, rec->popShell,
				      NULL, 0);

    MakeLabel(w, rec->form, "    Filename:", NULL, NULL);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNstring, DEFAULT_RECORD_FILE);
    makeArg(XtNresize, XawtextResizeWidth);
    makeArg(XtNresizable, True);
    makeArg(XtNfromHoriz, w);
    rec->file = XtCreateManagedWidget("filename", asciiTextWidgetClass,
				      rec->form, args, i);
    XtOverrideTranslations(rec->file, translations);

    MakeLabel(w, rec->form, "Max Duration:", NULL, rec->file);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNright, XawChainLeft);
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNstring, DEFAULT_RECORD_DURATION);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, rec->file);
    rec->duration = XtCreateManagedWidget("duration", asciiTextWidgetClass,
					  rec->form, args, i);
    XtOverrideTranslations(rec->duration, translations);

    i = 0;
    Anchor();
    makeArg(XtNfromVert, rec->file);
    makeArg(XtNfromHoriz, rec->duration);
    rec->readOnly = XtCreateManagedWidget("Read Only", toggleWidgetClass,
					  rec->form, args, i);

    MakeLabel(w, rec->form, "   Frequency:", NULL, rec->readOnly);

    i = 0;
    Anchor();
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNstring, DEFAULT_RECORD_RATE);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, rec->readOnly);
    rec->rate = XtCreateManagedWidget("rate", asciiTextWidgetClass,
				      rec->form, args, i);
    XtOverrideTranslations(rec->rate, translations);

    i = 0;
    Anchor();
    makeArg(XtNfromVert, rec->readOnly);
    makeArg(XtNfromHoriz, rec->rate);
    rec->mode = XtCreateManagedWidget("Mic", toggleWidgetClass, rec->form,
				      args, i);
    XtAddCallback(rec->mode, XtNcallback, modeCB, globals);

    MakeLabel(w, rec->form, "      Format:", NULL, rec->mode);

    rec->menu = XtCreatePopupShell("menu", simpleMenuWidgetClass, rec->form,
				   NULL, 0);

    i = 0;
    makeArg(XtNcallback, menuCallbacks);
    for (j = 0; j < AuServerNumFormats(globals->aud); j++)
	if (SoundToSndFormat(AuServerFormat(globals->aud, j)) !=
	    SND_FORMAT_UNSPECIFIED)
	    XtCreateManagedWidget(AuFormatToString(AuServerFormat(globals->aud,
								  j)),
				  smeBSBObjectClass, rec->menu, args, i);

    for (j = 0; j < AuServerNumFormats(globals->aud); j++)
	if (SoundToSndFormat(AuServerFormat(globals->aud, j)) !=
	    SND_FORMAT_UNSPECIFIED)
	{
	    s = AuFormatToString(AuServerFormat(globals->aud, j));
	    break;
	}

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNjustify, XtJustifyLeft);
    makeArg(XtNresizable, True);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, rec->mode);
    rec->menuButton = XtCreateManagedWidget(s, menuButtonWidgetClass,
					    rec->form, args, i);

    MakeLabel(w, rec->form, "Comment:", NULL, rec->menuButton);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNresize, XawtextResizeWidth);
    makeArg(XtNresizable, True);
    makeArg(XtNfromVert, w);
    rec->comment = XtCreateManagedWidget("comment", asciiTextWidgetClass,
					 rec->form, args, i);
    XtOverrideTranslations(rec->comment, translations);

    i = 0;
    Anchor();
    makeArg(XtNfromVert, rec->comment);
    sprintf(buf, GAIN_FORMAT, 0);
    makeArg(XtNlabel, buf);
    rec->gainLabel = XtCreateManagedWidget("VolumeLabel", labelWidgetClass,
					   rec->form, args, i);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNfromHoriz, rec->gainLabel);
    makeArg(XtNfromVert, rec->comment);
    makeArg(XtNorientation, XtorientHorizontal);
    makeArg(XtNresizable, True);
    rec->gainBar = XtCreateManagedWidget("GainBar", scrollbarWidgetClass,
					 rec->form, args, i);
    XtAddCallback(rec->gainBar, XtNscrollProc, gainScrollCB, globals);
    XtAddCallback(rec->gainBar, XtNjumpProc, gainJumpCB, globals);

    MakeCommandButton(rec->record, rec->form, "Record",
		      NULL, rec->gainLabel, recordStartCB);
    MakeCommandButton(rec->monitor, rec->form, "Monitor",
		      rec->record, rec->gainLabel, monitorCB);
    MakeCommandButton(rec->new, rec->form, "New Bucket",
		      rec->monitor, rec->gainLabel, newBucketCB);
    MakeCommandButton(rec->dismiss, rec->form, "Dismiss",
		      rec->new, rec->gainLabel, NULL);

    XtRealizeWidget(rec->popShell);

    XtVaGetValues(rec->dismiss, XtNx, &x, XtNwidth, &width, NULL);
    rightX = x + width - 5;

    XtVaGetValues(rec->record, XtNx, &x, NULL);
    XtVaSetValues(rec->comment, XtNwidth, rightX - x, NULL);
    XtVaGetValues(rec->file, XtNx, &x, NULL);
    XtVaSetValues(rec->file, XtNwidth, rightX - x, NULL);
    XtVaGetValues(rec->gainBar, XtNx, &x, NULL);
    XtVaGetValues(rec->gainLabel, XtNheight, &height, NULL);
    XtVaSetValues(rec->gainBar, XtNwidth, rightX - x, XtNheight, height, NULL);
}

int
main(argc, argv)
int             argc;
char          **argv;
{
    int             i,
                    endian,
                    audioEventBase,
                    audioErrorBase;
    Dimension       infoWidth,
                    labelHeight;
    Arg             args[10];
    char            buf[BUF_SIZE],
                   *audioServerString = NULL,
                   *dir = NULL;
    GlobalDataRec   globalData,
                   *globals;
    Position        infoX,
                    x;
    XtAppContext    appContext;
    Widget          w;
    XtTranslations  translations;
    extern int      AuMonitorFormat;
    static XtActionsRec Actions[] = {"Ok", okAction};

    globals = &globalData;

    globals->topLevel = XtVaAppInitialize(&appContext, APP_CLASS, NULL, ZERO,
					  &argc, argv, defaultResources,
					  NULL, 0);

    globals->volume = DEFAULT_VOLUME;
    globals->numFiles = 0;
    globals->dpy = XtDisplay(globals->topLevel);

    while (argc > 1)
    {
	argv++;
	argc--;

	if (!strcmp("-a", *argv) || !strcmp("-audio", *argv))
	{
	    audioServerString = argv[1];
	    argv++;
	    argc--;
	    if (!argc)
	    {
		printf("usage: audemo [ -toolkitoption ...] [-audio AUDIOSERVER] [directory]\n");
		exit(1);
	    }
	}
	else
	    dir = *argv;
    }

    if (!dir)
	dir = (char *) getcwd(NULL, 256);

    if (!(globals->aud = AuOpenServer(audioServerString, 0, NULL,
				      0, NULL, NULL)))
	fatalError("Can't connect to audio server");

    for (i = 0; i < AuServerNumDevices(globals->aud); i++)
	if ((AuDeviceKind(AuServerDevice(globals->aud, i)) ==
	     AuComponentKindPhysicalInput))
	{
	    globals->inputDeviceId =
		AuDeviceIdentifier(AuServerDevice(globals->aud, i));
	    break;
	}

    translations = XtParseTranslationTable(TRANS);
    XtAppAddActions(appContext, Actions, XtNumber(Actions));

    globals->form = XtCreateManagedWidget("form", formWidgetClass,
					  globals->topLevel, NULL, 0);

    MakeCommandButton(globals->play, globals->form, "Play", NULL, NULL, playCB);
    MakeCommandButton(globals->record, globals->form, "Record", globals->play,
		      NULL, recordCB);
    MakeCommandButton(globals->buckets, globals->form, "Buckets",
		      globals->record, NULL, bucketsCB);
    i = 0;
    Anchor();
    makeArg(XtNfromHoriz, globals->buckets);
    globals->meter = XtCreateManagedWidget("Meter", toggleWidgetClass,
					   globals->form, args, i);
    XtAddCallback(globals->meter, XtNcallback, meterToggleCB, globals);

    MakeCommandButton(globals->rescan, globals->form, "Rescan",
		      globals->meter, NULL, rescanCB);
    MakeCommandButton(globals->quit, globals->form, "Quit", globals->rescan,
		      NULL, quitCB);

    sprintf(buf, "NCDaudio %d.%d", AuServerProtocolMajorVersion(globals->aud),
	    AuServerProtocolMinorVersion(globals->aud));
    MakeLabel(w, globals->form, buf, globals->quit, NULL);

    MakeLabel(w, globals->form, " Left: ", NULL, w);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, globals->play);
    makeArg(XtNorientation, XtorientHorizontal);
    makeArg(XtNresizable, True);
    makeArg(XtNsensitive, False);
    makeArg(XtNminimumThumb, 0);
    globals->leftMeter = XtCreateManagedWidget("leftMeter",
					       scrollbarWidgetClass,
					       globals->form, args, i);

    MakeLabel(w, globals->form, "Right: ", NULL, globals->leftMeter);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, globals->leftMeter);
    makeArg(XtNorientation, XtorientHorizontal);
    makeArg(XtNresizable, True);
    makeArg(XtNsensitive, False);
    makeArg(XtNminimumThumb, 0);
    globals->rightMeter = XtCreateManagedWidget("rightMeter",
						scrollbarWidgetClass,
						globals->form, args, i);

    i = 0;
    Anchor();
    makeArg(XtNfromVert, w);
    sprintf(buf, VOLUME_FORMAT, globals->volume);
    makeArg(XtNlabel, buf);
    globals->volumeLabel = XtCreateManagedWidget("VolumeLabel",
						 labelWidgetClass,
						 globals->form, args, i);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNfromHoriz, globals->volumeLabel);
    makeArg(XtNfromVert, w);
    makeArg(XtNorientation, XtorientHorizontal);
    makeArg(XtNresizable, True);
    globals->volumeBar = XtCreateManagedWidget("VolumeBar",
					       scrollbarWidgetClass,
					       globals->form, args, i);
    XawScrollbarSetThumb(globals->volumeBar,
			 ((float) DEFAULT_VOLUME) / MAX_VOLUME, -1.0);
    XtAddCallback(globals->volumeBar, XtNscrollProc, scrollProcCB, globals);
    XtAddCallback(globals->volumeBar, XtNjumpProc, jumpProcCB, globals);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNstring, HELP_INFO);
    makeArg(XtNscrollVertical, XawtextScrollWhenNeeded);
    makeArg(XtNwrap, XawtextWrapWord);
    makeArg(XtNdisplayCaret, False);
    makeArg(XtNfromVert, globals->volumeLabel);
    globals->info = XtCreateManagedWidget("info", asciiTextWidgetClass,
					  globals->form, args, i);

    i = 0;
    infoWidth = getFontCharWidth(globals->info) * INFO_COLS;
    makeArg(XtNwidth, infoWidth);
    makeArg(XtNheight, getFontCharHeight(globals->info) * INFO_ROWS);
    XtSetValues(globals->info, args, i);

    MakeLabel(w, globals->form, "Directory: ", NULL, globals->info);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNstring, dir);
    makeArg(XtNresize, XawtextResizeWidth);
    makeArg(XtNresizable, True);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, globals->info);
    globals->dir = XtCreateManagedWidget("directory", asciiTextWidgetClass,
					 globals->form, args, i);
    XtOverrideTranslations(globals->dir, translations);

    MakeLabel(w, globals->form, "Template:  ", NULL, globals->dir);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNbottom, XawChainTop);
    makeArg(XtNleft, XawChainLeft);
    makeArg(XtNeditType, XawtextEdit);
    makeArg(XtNstring, DEFAULT_TEMPLATE);
    makeArg(XtNresize, XawtextResizeWidth);
    makeArg(XtNresizable, True);
    makeArg(XtNfromHoriz, w);
    makeArg(XtNfromVert, globals->dir);
    globals->template = XtCreateManagedWidget("template", asciiTextWidgetClass,
					      globals->form, args, i);
    XtOverrideTranslations(globals->template, translations);

    i = 0;
    makeArg(XtNtop, XawChainTop);
    makeArg(XtNfromVert, globals->template);
    makeArg(XtNallowVert, True);
    makeArg(XtNwidth, infoWidth);
    globals->view = XtCreateManagedWidget("view", viewportWidgetClass,
					  globals->form, args, i);

    globals->samples = XtCreateManagedWidget("samples", listWidgetClass,
					     globals->view, NULL, 0);
    XtAddCallback(globals->samples, XtNcallback, samplesCB, globals);

    XtVaSetValues(globals->view, XtNheight,
		  getFontCharHeight(globals->samples) * SAMPLES_ROWS, NULL);

    XtRealizeWidget(globals->topLevel);

    XtVaGetValues(globals->info, XtNx, &infoX, NULL);
    i = infoX + infoWidth;
    XtVaGetValues(globals->volumeBar, XtNx, &x, NULL);
    XtVaGetValues(globals->volumeLabel, XtNheight, &labelHeight, NULL);
    XtVaSetValues(globals->volumeBar, XtNwidth, i - x,
		  XtNheight, labelHeight, NULL);
    XtVaGetValues(globals->dir, XtNx, &x, NULL);
    XtVaSetValues(globals->dir, XtNwidth, i - x, NULL);
    XtVaGetValues(globals->template, XtNx, &x, NULL);
    XtVaSetValues(globals->template, XtNwidth, i - x, NULL);
    XtVaGetValues(globals->leftMeter, XtNx, &x, NULL);
    XtVaSetValues(globals->leftMeter, XtNwidth, i - x, NULL);
    XtVaGetValues(globals->rightMeter, XtNx, &x, NULL);
    XtVaSetValues(globals->rightMeter, XtNwidth, i - x, NULL);

    makeRecordDialog(globals);
    makeBucketDialog(globals);

    rescanCB(NULL, globals, NULL);

    endian = 1;
    AuMonitorFormat = *(char *) &endian ? AuFormatLinearSigned16LSB :
	AuFormatLinearSigned16MSB;

    AuXtAppAddAudioHandler(appContext, globals->aud);
    XtAppMainLoop(appContext);
}

#endif						/* XT */

static void
fatalError(message, arg)
char           *message,
               *arg;
{
    fprintf(stderr, message, arg);
    fprintf(stderr, ".\n");
    exit(1);
}

static int
sortRoutine(a, b)
char          **a,
              **b;
{
    return strcmp(strrchr(*a, '/') + 1, strrchr(*b, '/') + 1);
}

static char   **
makeFileList(fileNames, nfiles)
char          **fileNames;
int             nfiles;
{
    char          **fileList,
                   *p;
    int             i;

    qsort(fileNames, nfiles, sizeof(char *), sortRoutine);

    if (!(fileList = (char **) malloc(sizeof(char *) * nfiles)))
	fatalError("Can't malloc file list in makeFileList");

    for (i = 0; i < nfiles; i++)
    {
	p = strrchr(fileNames[i], '/') + 1;
	fileList[i] = (char *) strdup(p);

	if (p = strchr(fileList[i], '.'))
	    *p = 0;

	fileList[i] = (char *) realloc(fileList[i], strlen(fileList[i]) + 1);

	if (!fileList[i])
	    fatalError("Can't realloc file list in makeFileList");
    }

    return fileList;
}

static FILE    *
startFind(dir, template)
char           *dir,
               *template;
{
    char           *cmd,
                   *p;
    FILE           *fp;
    int             first = 1;

    cmd = (char *) malloc(2000);

    sprintf(cmd, "find %s -type f -a \\( ", dir);

    p = strtok(template, " ");

    while (p)
    {
	if (!first)
	    strcat(cmd, " -o -name '");
	else
	{
	    strcat(cmd, " -name '");
	    first = 0;
	}

	strcat(cmd, p);
	strcat(cmd, "'");

	p = strtok(NULL, " ");
    }

    strcat(cmd, " \\) -print");

    fp = popen(cmd, "r");
    free(cmd);
    return fp;
}

static int
getFileNames(dir, fileNames, template)
char           *dir,
             ***fileNames,
               *template;
{
    int             files,
                    rem;
    FILE           *fp;
    char            line[BUF_SIZE];

    /* prime the realloc pump */
    *fileNames = (char **) malloc(1);

    if (!fileNames)
	fatalError("Can't malloc file names in getFileNames");

    if (!(fp = startFind(dir, template)))
    {
	free(*fileNames);
	return 0;
    }

    files = 0;

    while (fgets(line, BUF_SIZE, fp))
    {
	*fileNames =
	    (char **) realloc(*fileNames, sizeof(char *) * (files + 1));

	if (!*fileNames)
	    fatalError("Can't realloc file names in getFileNames");

	line[strlen(line) - 1] = 0;	/* zap the trailing newline */
	(*fileNames)[files++] = (char *) strdup(line);
    }

    pclose(fp);

    if (!files)
	free(*fileNames);

    return files;
}
