/*
 * Copyright 1993 Greg Renda and Network Computing Devices, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this 
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED `AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * Author:
 *		Greg Renda
 *		greg@ncd.com
 *
 *		Network Computing Devices, Inc
 *		350 North Bernardo Ave
 *		Mountain View, CA 94043
 *
 * $NCDId: @(#)sndtoinclude.c,v 1.8 1993/03/02 03:16:17 lemke Exp $
 */

#include <stdio.h>
#include <string.h>
#include <audio/sound.h>

static struct
{
    int             format;
    char           *string;
}               formats[] =

{
    SND_FORMAT_ULAW_8, "SND_FORMAT_ULAW_8",
    SND_FORMAT_LINEAR_8, "SND_FORMAT_LINEAR_8",
    SND_FORMAT_LINEAR_16, "SND_FORMAT_LINEAR_16",
    SND_FORMAT_LINEAR_24, "SND_FORMAT_LINEAR_24",
    SND_FORMAT_LINEAR_32, "SND_FORMAT_LINEAR_32",
    SND_FORMAT_FLOAT_32, "SND_FORMAT_FLOAT_32",
    SND_FORMAT_FLOAT_64, "SND_FORMAT_FLOAT_64",
    SND_FORMAT_ULAW_COMPRESSED_8, "SND_FORMAT_ULAW_COMPRESSED_8"
};

static char           *
sndFormatDefine(format)
unsigned int    format;
{
    int             i;

    for (i = 0; i < sizeof(formats) / sizeof(formats[0]); i++)
	if (formats[i].format == format)
	    return formats[i].string;

    return "SND_FORMAT_UNKNOWN";
}

static void
fatalError(message, arg)
char           *message, *arg;
{
    fprintf(stderr, message, arg);
    fprintf(stderr, ".\n");
    exit(1);
}

static char	*
convert_quotes(str)
    char	*str;
{
    int	len = strlen(str) * 2 + 1;	/* more then enough for string */
    char	*new, *d, *s;

    s = str;
    d = new = (char *)malloc(len);
    if (!new)
	return new;
    while (*s) {
	if (*s == '"')
	    *d++ = '\\';
	*d++ = *s++;
    }
    free(str);
    return new;
}

int
main(argc, argv)
int argc;
char **argv;
{
    FILE *fp;
    SoundHeader    hdr;
    char *p, *name;
    char	*comment;
    unsigned int bytesPerSample, numSamples, i;
    int	spit_header = 0;

    if (argc > 2 && (!strncmp(argv[1], "-h", 2))) {
	spit_header = 1;
	name = argv[2];
    } else
	name = argv[1];

    if (!(fp = SoundOpenFileForReading(name, &hdr, &comment)))
	fatalError("Can't open snd file \"%s\"", name);

    if (p = strrchr(name, '.'))
	*p = 0;

    if (p = strrchr(name, '/'))
	name = p + 1;

    bytesPerSample = SoundSizeofFormat(hdr.format);
    numSamples = hdr.dataSize / bytesPerSample;

#ifdef notdef
    printf("#define %sFormat %s /* %s */\n", name, sndFormatDefine(hdr.format),
	   SoundFormatToString(hdr.format));
#endif
    printf("#define %sSampleRate %d\n", name, hdr.sampleRate);
    printf("#define %sNumTracks %d\n", name, hdr.tracks);
    printf("#define %sNumSamples %d\n", name, numSamples);
    p = convert_quotes(comment);
    printf("\nstatic char *%sComment = \"%s\";\n", name, p);
    free(p);

    /* spit out header */
    if (spit_header) {
	printf("\nstatic SoundHeader %sHeader = {\n", name);
	printf("    %20ld,\t /* magic */\n", hdr.sound_type);
	printf("    %20ld,\t /* format */\n", hdr.format);
	printf("    %20ld,\t /* dataOffset */\n", sizeof(SoundHeader));
	printf("    %20ld,\t /* dataSize */\n", hdr.dataSize);
	printf("    %20ld,\t /* sampleRate */\n", hdr.sampleRate);
	printf("    %20ld,\t /* tracks */\n", hdr.tracks);
	printf("    %20ld,\t /* bitsPerSample */\n", bytesPerSample);
	printf("    %20ld,\t /* swapped */\n", 0);
	printf("    %20ld,\t /* is_file */\n", 0);
	/* note we toss the comment in this header */
	printf("};\n\n");
    }


    if (bytesPerSample == 2)
    {
	unsigned short sample;

	printf("static unsigned short %sSamples[] =\n{", name);

	for (i = 0; i < numSamples; i++)
	{
	    if (!(i % 8))
		printf("\n    ");

	    fread(&sample, 1, bytesPerSample, fp);
	    printf("0x%04x, ", sample);
	}
	
	printf("\n};\n");
    }
    else if (bytesPerSample == 1)
    {
	unsigned char sample;

	printf("static unsigned char %sSamples[] =\n{", name);

	for (i = 0; i < numSamples; i++)
	{
	    if (!(i % 8))
		printf("\n    ");

	    fread(&sample, 1, bytesPerSample, fp);
	    printf("0x%02x, ", sample);
	}
	
	printf("\n};\n");
    }

    fclose(fp);
    exit(0);
}
