/*
 * Copyright 1993 Network Computing Devices, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * $NCDId: @(#)Util.c,v 1.10 1993/04/06 01:47:15 jim Exp $
 */

#include "Alibint.h"

int             AuMonitorRate = 10,
                AuMonitorFormat = AuFormatLinearSigned16MSB;

AuFlowID
AuGetScratchFlow(aud, ret_status)
AuServer       *aud;
AuStatus       *ret_status;
{
    AuFlowID        flow;
    int             i;

    if (aud->scratch_flows.num_inuse == AU_MAX_SCRATCH_FLOWS)
	return AuCreateFlow(aud, ret_status);

    for (i = 0; i < aud->scratch_flows.total; i++)
	if (!aud->scratch_flows.flows[i].inuse)
	{
	    aud->scratch_flows.flows[i].inuse = AuTrue;
	    aud->scratch_flows.num_inuse++;
	    return aud->scratch_flows.flows[i].flow;
	}

    if ((flow = AuCreateFlow(aud, ret_status)) != AuNone)
    {
	aud->scratch_flows.flows[aud->scratch_flows.total].flow = flow;
	aud->scratch_flows.flows[aud->scratch_flows.total].inuse = AuTrue;
	aud->scratch_flows.total++;
	aud->scratch_flows.num_inuse++;
    }

    return flow;
}

AuFlowID
AuGetScratchFlowToBucket(aud, bucket, import, ret_status)
AuServer       *aud;
AuBucketID      bucket;
int            *import;
AuStatus       *ret_status;
{
    AuElement       elements[2];
    AuFlowID        flow;
    AuBucketAttributes *ba;

    if (!(flow = AuGetScratchFlow(aud, ret_status)))
	return NULL;

    if (!(ba = AuGetBucketAttributes(aud, bucket, ret_status)))
    {
	AuReleaseScratchFlow(aud, flow, ret_status);
	return NULL;
    }

    AuMakeElementImportClient(&elements[0], AuBucketSampleRate(ba),
			      AuBucketFormat(ba), AuBucketNumTracks(ba),
			      AuTrue, 0, 0, 0, NULL);
    AuMakeElementExportBucket(&elements[1], 0, bucket,
			      AuBucketNumSamples(ba), 0, 0, NULL);

    /* set up the flow */
    AuSetElements(aud, flow, AuFalse, 2, elements, ret_status);

    *import = 0;
    AuFreeBucketAttributes(aud, 1, ba);
    return flow;
}

AuFlowID
AuGetScratchFlowFromBucket(aud, bucket, export, ret_status)
AuServer       *aud;
AuBucketID      bucket;
int            *export;
AuStatus       *ret_status;
{
    AuElement       elements[2];
    AuFlowID        flow;
    AuBucketAttributes *ba;

    if (!(flow = AuGetScratchFlow(aud, ret_status)))
	return NULL;

    if (!(ba = AuGetBucketAttributes(aud, bucket, ret_status)))
    {
	AuReleaseScratchFlow(aud, flow, ret_status);
	return NULL;
    }

    AuMakeElementImportBucket(&elements[0], AuBucketSampleRate(ba), bucket,
			      AuBucketNumSamples(ba), 0, 0, NULL);
    AuMakeElementExportClient(&elements[1], 0, AuBucketSampleRate(ba),
			      AuBucketFormat(ba), AuBucketNumTracks(ba),
			      AuTrue, 0, 0, 0, NULL);

    /* set up the flow */
    AuSetElements(aud, flow, AuFalse, 2, elements, ret_status);

    *export = 1;
    AuFreeBucketAttributes(aud, 1, ba);
    return flow;
}

void
AuReleaseScratchFlow(aud, flow, ret_status)
AuServer       *aud;
AuFlowID        flow;
AuStatus       *ret_status;
{
    int             i;

    for (i = 0; i < aud->scratch_flows.total; i++)
	if (aud->scratch_flows.flows[i].flow == flow)
	{
	    aud->scratch_flows.flows[i].inuse = AuFalse;
	    aud->scratch_flows.num_inuse--;
	    return;
	}

    AuDestroyFlow(aud, flow, ret_status);
}

void
AuStopFlow(aud, flow, ret_status)
AuServer       *aud;
AuFlowID        flow;
AuStatus       *ret_status;
{
    AuElementState  states[1];

    AuMakeElementState(&states[0], flow, AuElementAll, AuStateStop);
    AuSetElementStates(aud, 1, states, ret_status);
}

static struct
{
    int             format;
    char           *string;
}               formats[] =
{
    AuFormatULAW8, "8-bit uLAW",
    AuFormatLinearUnsigned8, "8-bit unsigned linear",
    AuFormatLinearSigned8, "8-bit signed linear",
    AuFormatLinearSigned16MSB, "16-bit signed linear (big endian)",
    AuFormatLinearUnsigned16MSB, "16-bit unsigned linear (big endian)",
    AuFormatLinearSigned16LSB, "16-bit signed linear (little endian)",
    AuFormatLinearUnsigned16LSB, "16-bit unsigned linear (little endian)",
};

_AuConst char  *
AuFormatToString(format)
unsigned int    format;
{
    int             i;

    for (i = 0; i < sizeof(formats) / sizeof(formats[0]); i++)
	if (formats[i].format == format)
	    return formats[i].string;

    return "Unknown";
}

int
AuStringToFormat(s)
_AuConst char  *s;
{
    int             i;

    for (i = 0; i < sizeof(formats) / sizeof(formats[0]); i++)
	if (!strcasecmp(s, formats[i].string))
	    return formats[i].format;

    return -1;
}

typedef struct
{
    AuFlowID        flow;
    AuPointer       callback_data;
    void            (*callback) (
#if NeedNestedPrototypes 
				 AuServer *, AuEventHandlerRec *,
				 AuEvent *, AuPointer
#endif
				 );
}               MonitorDeviceRec, *MonitorDevicePtr;

static          AuBool
EventHandler(aud, ev, handler)
AuServer       *aud;
AuEvent        *ev;
AuEventHandlerRec *handler;
{
    MonitorDevicePtr priv = (MonitorDevicePtr) handler->data;

    switch (ev->type)
    {
	case AuEventTypeMonitorNotify:
	    if (priv->callback)
		(*priv->callback) (aud, handler, ev, priv->callback_data);
	    break;
	case AuEventTypeElementNotify:
	    {
		AuElementNotifyEvent *event = (AuElementNotifyEvent *) ev;

		if (event->kind == AuElementNotifyKindState &&
		    event->cur_state == AuStateStop)
		{
		    if (priv->callback)
			(*priv->callback) (aud, handler, ev,
					   priv->callback_data);

		    AuUnregisterEventHandler(aud, handler);
		    AuReleaseScratchFlow(aud, priv->flow, NULL);
		    Aufree(priv);
		}
		break;
	    }
    }

    return AuTrue;
}

AuEventHandlerRec *
AuMonitorDevice(aud, rate, in_device, out_device, volume, callback,
		callback_data, pflow, pmultelem, pmonelem, ret_status)
AuServer       *aud;
int             rate;
AuDeviceID      in_device,
                out_device;
AuFixedPoint    volume;
void            (*callback) (
#if NeedFunctionPrototypes
			     AuServer *, AuEventHandlerRec *,
			     AuEvent *, AuPointer
#endif
			     );
AuPointer       callback_data;
AuFlowID       *pflow;
int            *pmultelem,
               *pmonelem;
AuStatus       *ret_status;
{
    AuElement       elements[4];
    AuElementState  states[1];
    AuElementAction actions[1];
    AuEventHandlerRec *handler;
    MonitorDevicePtr priv;
    int             numTracks,
                    i;

    if (!(priv = (MonitorDevicePtr) Aumalloc(sizeof(MonitorDeviceRec))))
	return NULL;

    priv->callback = callback;
    priv->callback_data = callback_data;

    if ((priv->flow = AuGetScratchFlow(aud, NULL)) == AuNone)
    {
	Aufree(priv);
	return NULL;
    }

    if (out_device == AuNone)
    {
	AuDeviceAttributes *d = NULL;

	/* look up the input device attributes */
	for (i = 0; i < AuServerNumDevices(aud); i++)
	{
	    if (AuDeviceIdentifier(AuServerDevice(aud, i)) == in_device)
	    {
		d = AuServerDevice(aud, i);
		break;
	    }
	}

	if (!d)
	{
	    Aufree(priv);
	    return NULL;
	}

	numTracks = AuDeviceNumTracks(d);

	/* look for a matching output device */
	for (i = 0; i < AuServerNumDevices(aud); i++)
	    if ((AuDeviceKind(AuServerDevice(aud, i)) ==
		 AuComponentKindPhysicalOutput) &&
		AuDeviceNumTracks(AuServerDevice(aud, i)) == numTracks)
	    {
		out_device = AuDeviceIdentifier(AuServerDevice(aud, i));
		break;
	    }

	if (out_device == AuNone)
	{
	    Aufree(priv);
	    return NULL;
	}
    }
    else
    {
	AuDeviceAttributes *d;

	if (!(d = AuGetDeviceAttributes(aud, out_device, ret_status)))
	{
	    Aufree(priv);
	    return NULL;
	}

	numTracks = AuDeviceNumTracks(d);
	AuFreeDeviceAttributes(aud, 1, d);
    }

    AuMakeSendNotifyAction(&actions[0], AuStateStop, AuStateAny, AuReasonAny);

    AuMakeElementImportDevice(&elements[0], rate, in_device, AuUnlimitedSamples,
			      1, actions);
    AuMakeElementMultiplyConstant(&elements[1], 0, volume);
    AuMakeElementExportDevice(&elements[2], 1, out_device, rate,
			      AuUnlimitedSamples, 0, NULL);

    if (pmonelem)
    {
	i = 4;
	*pmonelem = 3;
	AuMakeElementExportMonitor(&elements[3], 0, AuMonitorRate,
				   AuMonitorFormat, numTracks);
    }
    else
	i = 3;

    /* set up the flow */
    AuSetElements(aud, priv->flow, AuTrue, i, elements, ret_status);

    if (!(handler = AuRegisterEventHandler(aud, AuEventHandlerIDMask,
					0, priv->flow, EventHandler, priv)))
    {
	AuReleaseScratchFlow(aud, priv->flow, ret_status);
	Aufree(priv);
	return NULL;
    }

    /* start up the components */
    AuMakeElementState(&states[0], priv->flow, AuElementAll, AuStateStart);
    AuSetElementStates(aud, 1, states, ret_status);

    if (pflow)
	*pflow = priv->flow;

    if (pmultelem)
	*pmultelem = 1;

    return handler;
}
