/*
 * Copyright 1993 Network Computing Devices, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * $NCDId: @(#)Xtutil.c,v 1.6 1993/01/29 20:47:31 lemke Exp $
 * 
 * This file contains utilities for writing Xt programs.
 */

#include <X11/Intrinsic.h>
#include <audio/Alibint.h>
#include "Xtutil.h"

typedef struct
{
    AuServer       *aud;
    XtAppContext    app_context;
    AuBool          workProcActive;
} PrivSyncHandlerData;


/* ARGSUSED */
void
AuXtHandleAudioEvents(closure, pfd, pinputid)
XtPointer       closure;
int            *pfd;
XtInputId      *pinputid;
{
    AuServer       *aud = (AuServer *) closure;

    AuHandleEvents(aud);
}


static          Boolean
_au_xt_workproc(data)
XtPointer	data;
{
    PrivSyncHandlerData * priv = (PrivSyncHandlerData *) data;
    AuFlush(priv->aud);
    priv->workProcActive = AuFalse;
    return True;
}

/* ARGSUSED */
static void
_au_xt_synchandler(aud, handler, data)
AuServer       *aud;
AuSyncHandlerRec *handler;
XtPointer data;
{
    PrivSyncHandlerData * priv = (PrivSyncHandlerData *) data;
    if (!priv->workProcActive)
    {
	XtAppAddWorkProc(priv->app_context, _au_xt_workproc, priv);
	priv->workProcActive = AuTrue;
    }
}

XtInputId
AuXtAppAddAudioHandler(app_context, aud)
XtAppContext    app_context;
AuServer       *aud;
{
    PrivSyncHandlerData * priv;

    if (!(priv = (PrivSyncHandlerData*)Aumalloc(sizeof(PrivSyncHandlerData))))
	return 0;

    priv->app_context = app_context;
    priv->aud = aud;
    priv->workProcActive = AuFalse;

    if (!AuRegisterSyncHandler(aud, _au_xt_synchandler, (XtPointer) priv))
    {
	Aufree(priv);
	return 0;
    }

    return XtAppAddInput(app_context, AuServerConnectionNumber(aud),
			 (XtPointer) XtInputReadMask, AuXtHandleAudioEvents,
			 (XtPointer) aud);
}
