/*
 * <audio/snd.h>
 * 
 * This file contains interfaces to the .snd files.  It is not dependent on the
 * NCD-AUDIO service and can easily be used for other purposes.
 *
 * Copyright 1993 Network Computing Devices, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * $NCDId: @(#)snd.h,v 1.8 1993/02/26 23:55:28 lemke Exp $
 */

#ifndef _SND_H_
#define _SND_H_

#include <stdio.h>

/*
 * If we are being used outside the NCD-AUDIO sound library, then we'll need
 * some ANSIfication definitions.
 */
#ifndef NeedFunctionPrototypes
#if defined(FUNCPROTO) || __STDC__ || defined(__cplusplus) || defined(c_plusplus)
#define NeedFunctionPrototypes 1
#else
#define NeedFunctionPrototypes 0
#endif
#endif						/* NeedFunctionPrototypes */

#ifndef _SndConst
#if __STDC__ || defined(__cplusplus) || defined(c_plusplus) || (FUNCPROTO&4)
#define _SndConst const
#else
#define _SndConst				/**/
#endif
#endif						/* _SndConst */


/*****************************************************************************
 *				  DATA TYPES				     *
 *****************************************************************************/

#define SND_MAGIC				0x2e736e64	/* .snd */
#define SND_DATA_SIZE_UNKNOWN			0xffffffff

#define SND_FORMAT_UNSPECIFIED			0
#define SND_FORMAT_ULAW_8			1
#define SND_FORMAT_LINEAR_8			2
#define SND_FORMAT_LINEAR_16			3
#define SND_FORMAT_LINEAR_24			4
#define SND_FORMAT_LINEAR_32			5
#define SND_FORMAT_FLOAT_32			6
#define SND_FORMAT_FLOAT_64			7
#define SND_FORMAT_ULAW_COMPRESSED_8		23

typedef struct _SndHeader
{
    unsigned long   magic;			/* the value 0x2e736e64
						 * (ASCII ".snd") */
    unsigned long   dataOffset;			/* the offset, in octets, to
						 * the data part. The minimum
						 * valid number is 24
						 * (decimal). */
    unsigned long   dataSize;			/* the size in octets, of the
						 * data part. If unknown, the
						 * value 0xffffffff should be
						 * used. */
    /** the data format:
     *
     * value    format
     *   1      8-bit ISDN u-law
     *   2      8-bit linear PCM [REF-PCM]
     *   3      16-bit linear PCM
     *   4      24-bit linear PCM
     *   5      32-bit linear PCM
     *   6      32-bit IEEE floating point
     *   7      64-bit IEEE floating point
     *  23      8-bit ISDN u-law compressed
     *          using the CCITT G.721 ADPCM
     *          voice data format scheme.
     */
    unsigned long   format;

    unsigned long   sampleRate;			/* the number of
						 * samples/second (e.g.,
						 * 8000) */
    unsigned long   tracks;			/* the number of interleaved
						 * tracks (e.g., 1) */
}               SndHeader;


/*****************************************************************************
 *			       PUBLIC INTERFACES			     *
 *****************************************************************************/


#define SndSeek(fp, hdr, offset, type) \
    fseek(fp, (type) == 0 ? (offset) + (hdr)->dataOffset : (offset), type)


extern FILE    *
SndOpenFileForReading(
#if NeedFunctionPrototypes
		      _SndConst char *,		/* file name */
		      SndHeader *,		/* fillin_header */
		      char **			/* ret_comment */
#endif
);

extern FILE    *
SndOpenFileForWriting(
#if NeedFunctionPrototypes
		      _SndConst char *,		/* file name */
		      SndHeader *,		/* header */
		      _SndConst char *		/* comment */
#endif
);

extern unsigned int 
SndSizeofFormat(
#if NeedFunctionPrototypes
		int				/* format */
#endif
);



extern _SndConst char *
SndFormatToString(
#if NeedFunctionPrototypes
		  unsigned int			/* format */
#endif
);

extern int 
SndStringToFormat(
#if NeedFunctionPrototypes
		  _SndConst char *		/* text */
#endif
);


#endif						/* _SND_H_ */
