/*
 * Copyright 1993 Network Computing Devices, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 *
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $NCDId: @(#)sound.c,v 1.9 1993/02/19 22:49:39 greg Exp $
 */

/*
 * generic sound handling routines
 */

#include	<stdio.h>
#include 	<strings.h>
#include	<audio/audio.h>
#include	<audio/sound.h>

int
WaveToSoundFormat(wh)
    WaveHeader *wh;
{
    switch (wh->wFormatTag) {
    case WAVE_FORMAT_PCM:
	if (wh->datasize == 8)
	    return AuFormatLinearUnsigned8;
	else if (wh->datasize == 16)
	    return (wh->swap ?
		    AuFormatLinearSigned16LSB : AuFormatLinearSigned16MSB);
	else
	    return -1;
    default:
	return -1;
    }
}

int
SoundToWaveFormat(sh)
    SoundHeader *sh;
{
    switch (sh->format) {
    default:
	return -1;
    case AuFormatLinearUnsigned8:
    case AuFormatLinearSigned16LSB:
    case AuFormatLinearSigned16MSB:
	return WAVE_FORMAT_PCM;
    }
}

int
SndToSoundFormat(fmt)
    int         fmt;
{
    switch (fmt) {
    case SND_FORMAT_ULAW_8:
	return AuFormatULAW8;
    case SND_FORMAT_LINEAR_8:
	return AuFormatLinearUnsigned8;
    case SND_FORMAT_LINEAR_16:
	return AuFormatLinearSigned16MSB;
    default:
	return AuNone;
    }
}

int
SoundToSndFormat(fmt)
    int         fmt;
{
    switch (fmt) {
    case AuFormatULAW8:
	return SND_FORMAT_ULAW_8;
    case AuFormatLinearUnsigned8:
	return SND_FORMAT_LINEAR_8;
    case AuFormatLinearSigned16MSB:
	return SND_FORMAT_LINEAR_16;
    default:
	return SND_FORMAT_UNSPECIFIED;
    }
}

static int
voc_to_sound(vh, soundh)
    VocHeader  *vh;
    SoundHeader *soundh;
{
    soundh->sound_type = VOC_MAGIC;
    soundh->format = AuFormatLinearUnsigned8;
    soundh->dataOffset = 26;
    soundh->dataSize = vh->datalen;
    soundh->sampleRate = vh->rate;
    soundh->tracks = vh->tracks;
    soundh->swapped = 1;
    soundh->bitsPerSample = vh->datasize;

    return 1;
}

static int
sound_to_voc(soundh, vh)
    SoundHeader *soundh;
    VocHeader  *vh;
{
    vh->datalen = soundh->dataSize;
    vh->rate = soundh->sampleRate;
    vh->tracks = soundh->tracks;
    vh->datasize = soundh->bitsPerSample;
    return 1;
}

static int
wave_to_sound(wh, soundh)
    WaveHeader *wh;
    SoundHeader *soundh;
{
    soundh->sound_type = WAVE_MAGIC;
    soundh->format = WaveToSoundFormat(wh);
    soundh->dataOffset = wh->data_start_offset;
    soundh->dataSize = wh->datalen;
    soundh->sampleRate = wh->dwSamplesPerSec;
    soundh->tracks = wh->wChannels;
    soundh->swapped = wh->swap;
    soundh->bitsPerSample = wh->datasize;

    return 1;
}

static int
sound_to_wave(soundh, wh)
    SoundHeader *soundh;
    WaveHeader *wh;
{
    wh->wFormatTag = SoundToWaveFormat(soundh);
    wh->data_start_offset = soundh->dataOffset;
    wh->datalen = soundh->dataSize;
    wh->dwSamplesPerSec = soundh->sampleRate;
    wh->wChannels = soundh->tracks;
    wh->swap = soundh->swapped;
    wh->datasize = soundh->bitsPerSample;

    wh->wBlockAlign = (wh->wChannels * wh->datasize + 7) / 8;
    wh->formatlen = WaveFormatLength(wh->wFormatTag);
    wh->dwAvgBytesPerSec =
	(wh->dwSamplesPerSec * wh->wChannels * wh->datasize + 7) / 8;

    return 1;
}

static int
snd_to_sound(sh, soundh)
    SndHeader  *sh;
    SoundHeader *soundh;
{
    soundh->sound_type = SND_MAGIC;
    soundh->format = SndToSoundFormat(sh->format);
    soundh->dataOffset = sh->dataOffset;
    soundh->dataSize = sh->dataSize;
    soundh->sampleRate = sh->sampleRate;
    soundh->tracks = sh->tracks;
    soundh->swapped = 0;	/* XXX */
    switch (sh->format) {
    case SND_FORMAT_ULAW_8:
    case SND_FORMAT_LINEAR_8:
    case SND_FORMAT_ULAW_COMPRESSED_8:
	soundh->bitsPerSample = 8;
	break;
    case SND_FORMAT_LINEAR_16:
	soundh->bitsPerSample = 16;
	break;
    case SND_FORMAT_LINEAR_24:
	soundh->bitsPerSample = 24;
	break;
    case SND_FORMAT_LINEAR_32:
    case SND_FORMAT_FLOAT_32:
	soundh->bitsPerSample = 32;
	break;
    case SND_FORMAT_FLOAT_64:
	soundh->bitsPerSample = 64;
	break;
    }

    return 1;
}

static int
sound_to_snd(soundh, sh)
    SoundHeader *soundh;
    SndHeader  *sh;
{
    sh->magic = SND_MAGIC;
    sh->format = SoundToSndFormat(soundh->format);
    sh->dataOffset = soundh->dataOffset;
    sh->dataSize = soundh->dataSize;
    sh->sampleRate = soundh->sampleRate;
    sh->tracks = soundh->tracks;

    return 1;
}

unsigned int
SoundSizeofFormat(format)
    int         format;
{
    static unsigned int f2bps[] =
    {
	0,			/* unused */
	1,			/* AuFormatULAW8 */
	1,			/* AuFormatLinearUnsigned8 */
	1,			/* AuFormatLinearSigned8 */
	2,			/* AuFormatLinearSigned16MSB */
	2,			/* AuFormatLinearUnsigned16MSB */
	2,			/* AuFormatLinearSigned16LSB */
	2,			/* AuFormatLinearUnsigned16LSB */
    };

    if (format < 0 || format > (sizeof(f2bps) / sizeof(f2bps[0]) - 1))
	format = 0;

    return f2bps[format];
}

FILE       *
SoundOpenFileForReading(fname, soundh, comment)
    _SoundConst char *fname;
    SoundHeader *soundh;
    char      **comment;
{
    FILE       *gs;
    WaveHeader  wh;
    SndHeader   sh;
    VocHeader   vh;

    soundh->is_file = strcmp(fname, "-");

    gs = SndOpenFileForReading(fname, &sh, comment);
    if (gs) {
	if (snd_to_sound(&sh, soundh))
	    return gs;
    }
    gs = WaveOpenFileForReading(fname, &wh, comment);
    if (gs) {
	if (wave_to_sound(&wh, soundh))
	    return gs;
    }
    gs = VocOpenFileForReading(fname, &vh, comment);
    if (gs) {
	if (voc_to_sound(&vh, soundh))
	    return gs;
    }
/* other formats go here */
    return (FILE *) 0;
}

FILE       *
SoundOpenFileForWriting(fname, soundh, comment)
    _SoundConst char *fname;
    SoundHeader *soundh;
    char       *comment;
{
    SndHeader   sh;
    WaveHeader  wh;
    VocHeader   vh;

    soundh->is_file = strcmp(fname, "-");

    if (soundh->sound_type == SND_MAGIC) {
	if (sound_to_snd(soundh, &sh))
	    return SndOpenFileForWriting(fname, &sh, comment);
    }
    if (soundh->sound_type == WAVE_MAGIC) {
	if (sound_to_wave(soundh, &wh))
	    return WaveOpenFileForWriting(fname, &wh, comment);
    }
    if (soundh->sound_type == VOC_MAGIC) {
	if (sound_to_voc(soundh, &vh))
	    return VocOpenFileForWriting(fname, &vh, comment);
    }
/* other formats go here */
    fprintf(stderr, "Unrecognized sound type %d\n", soundh->sound_type);
    return (FILE *) NULL;
}

void
SoundCloseFile(fp)
    FILE       *fp;
{
    if (!((fp == stdin) || (fp == stdout) || (fp == stderr)))
	fclose(fp);
}
