/*
 * Copyright 1993 Network Computing Devices, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * $NCDId: @(#)sound.h,v 1.8 1993/02/18 03:19:50 greg Exp $
 */

/*
 * generic sound file handling
 */

#ifndef _SOUND_H_
#define _SOUND_H_

#include	<audio/snd.h>
#include	<audio/wave.h>
#include	<audio/voc.h>

#ifndef NeedFunctionPrototypes

#if defined(FUNCPROTO) || __STDC__ || defined(__cplusplus) || defined(c_plusplus)
#define NeedFunctionPrototypes 1
#else
#define NeedFunctionPrototypes 0
#endif

#endif				/* NeedFunctionPrototypes */

#ifndef _SoundConst

#if __STDC__ || defined(__cplusplus) || defined(c_plusplus) || (FUNCPROTO&4)
#define _SoundConst const
#else
#define _SoundConst		/**/
#endif

#endif				/* _SoundConst */

typedef struct _SoundHeader {
    unsigned long sound_type;	/* SND_MAGIC, or .WAV */
    unsigned long format;	/* Au data format */
    unsigned long dataOffset;	/* offset to start of data from beginning of
				 * file */
    unsigned long dataSize;	/* size of sound data */
    unsigned long sampleRate;	/* samples per second */
    unsigned short tracks;
    unsigned short bitsPerSample;/* 8, 16, 32 */
    char swapped;   		/* data format */
    char is_file;   		/* data from file */
}           SoundHeader;


#define SoundSeek(fp, hdr, offset, type) 			\
    do { 							\
        if ((hdr)->is_file)					\
	    fseek(fp, (type) == 0 ? 				\
            	(offset) + (hdr)->dataOffset : (offset), type); \
    } while(0)



extern FILE *
SoundOpenFileForReading(
#if NeedFunctionPrototypes
		      _SoundConst char *,	/* file name */
		      SoundHeader *,		/* fillin_header */
		      char **			/* ret_comment */
#endif
);

extern FILE *
SoundOpenFileForWriting(
#if NeedFunctionPrototypes
			_SoundConst char *,	/* file name */
			SoundHeader *,		/* fillin_header */
			char *			/* ret_comment */
#endif
);

extern void
SoundCloseFile(
#if NeedFunctionPrototypes
			FILE	*
#endif
);

extern unsigned int
SoundSizeofFormat(
#if NeedFunctionPrototypes
			int			/* Au format */
#endif
);

#endif				/* _SOUND_H_ */
