/*
 * Copyright 1993 Network Computing Devices, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 *
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $NCDId: @(#)voc.c,v 1.6 1993/02/25 01:01:31 greg Exp $
 */

/*
 * .voc support files
 *
 * lots of stuff borrowed from sox
 */

#include	<stdio.h>
#include	<strings.h>
#include	<audio/voc.h>
#include	<audio/fileutil.h>

FILE       *
VocOpenFileForReading(fname, hdr, comment)
    _VocConst char *fname;
    VocHeader  *hdr;
    char      **comment;
{
    FILE       *fp;
    char        tbuf[32];
    int         endian = 1;
    short       version;
    int         swap = 0;
    int         code, hlen;
    int         sr,
                ctype,
                dlen;

    if (strcmp(fname, "-") == 0) {
	fp = stdin;
	/* XXX -- why is this needed?  otherwise we're somewhere else... */
	fseek(fp, 0, 0);
    } else {
	fp = fopen(fname, "r");
	if (!fp)
	    goto fail;
    }
    if (!fread(tbuf, 20, 1, fp)) {
	goto fail;
    }
    if (strncmp(tbuf, "Creative Voice File\032", 19))
	goto fail;

    swap = (*(char *) &endian == 0);
    hlen = FileReadS(fp, swap);
    if (hlen != 26)
	goto fail;

    /* version number */
    version = FileReadS(fp, swap);
    if (version != 0x10a)
	goto fail;
    /* checksum */
    version = FileReadS(fp, swap);
    if (version != 0x1129)
	goto fail;

    code = fgetc(fp);
    switch (code) {
    case VOC_DATA:
/* XXX -- probably crap for LSB machines */
	dlen = fgetc(fp);
	dlen += fgetc(fp) << 8;
	dlen += fgetc(fp) << 16;
	sr = fgetc(fp);
	ctype = fgetc(fp);		/* fetch and throw away */
#if defined(SABER)
	ctype = ctype;
#endif
	break;
    default:
	break;
    }

    if (comment) {
	*comment = FileCommentFromFilename(fname);
    }
    hdr->rate = 1000000.0 / (256 - sr);
    hdr->datasize = 8;
    hdr->tracks = 1;
    hdr->datalen = dlen;

    return fp;

fail:
    if (fp && strcmp(fname, "-"))
	fclose(fp);
    return (FILE *) 0;
}

/* XXX untested  -- do something with comment XXX */
/* ARGSUSED */
FILE       *
VocOpenFileForWriting(fname, hdr, comment)
    _VocConst char *fname;
    VocHeader  *hdr;
    char       *comment;
{
    FILE       *fp;
    int         swap;
    unsigned long tlen;
    int         endian = 1;
    int         sr;

    if (strcmp(fname, "-") == 0) {
	fp = stdout;
    } else {
	fp = fopen(fname, "w");
	if (!fp)
	    return fp;
    }
    swap = (*(char *) &endian != 0);
    (void) fwrite("Creative Voice File\032\032", 1, 20, fp);
    FileWriteS(26, fp, swap);
    /* version number & checksum */
    FileWriteS(0x10a, fp, swap);
    FileWriteS(0x1129, fp, swap);

    sr = 256 - (1000000.0 / (float) hdr->rate);
    fputc(VOC_DATA, fp);

    tlen = hdr->datalen;

    fputc(tlen, fp);
    fputc(tlen >> 8, fp);
    fputc(tlen >> 16, fp);
    fputc(sr, fp);
    fputc(0, fp);

    return fp;
}
