/*
 * Copyright 1993 Network Computing Devices, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 *
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $NCDId: @(#)wave.c,v 1.11 1993/03/02 23:42:32 lemke Exp $
 */

/*
 * WAVE support library
 */

#include	<stdio.h>
#include	<strings.h>
#include	<audio/wave.h>
#include	<audio/fileutil.h>

#if NeedFunctionPrototypes
int WaveFormatLength (unsigned short format)
#else
int
WaveFormatLength(format)
    unsigned short format;
#endif
{
    switch (format) {
    case WAVE_FORMAT_PCM:
	return 16;
    default:
	return -1;
    }
}

static int
verify_format(hdr)
    WaveHeader *hdr;
{
    if (WaveFormatLength(hdr->wFormatTag) < 0)
	return 0;
    return 1;
}

FILE       *
WaveOpenFileForReading(fname, hdr, comment)
    _WaveConst char *fname;
    WaveHeader *hdr;
    char      **comment;
{
    FILE       *fp;
    char        tbuf[32];
    int         swap;
    int         endian = 1;

    if (strcmp(fname, "-") == 0) {
	fp = stdin;
	/* XXX -- why is this needed?  otherwise we're somewhere else... */
	fseek(fp, 0, 0);
    } else {
	fp = fopen(fname, "r");
	if (!fp)
	    goto fail;
    }
    if (!fread(tbuf, 4, 1, fp)) {
	goto fail;
    }
    if (strncmp(tbuf, "RIFF", 4) == 0) {
	if (*(char *) &endian)
	    swap = 0;
	else
	    swap = 1;
    } else if (strncmp(tbuf, "RIFX", 4) == 0) {
	if (*(char *) &endian)
	    swap = 1;
	else
	    swap = 0;
    } else
	goto fail;
    hdr->swap = swap;

    (void) FileReadL(fp, swap);	/* eat length */

    if (!fread(tbuf, 4, 1, fp))
	goto fail;
/* XXX should really eat this and find next header */
    if (strncmp(tbuf, "WAVE", 4))
	goto fail;

    if (!fread(tbuf, 4, 1, fp))
	goto fail;
    if (strncmp(tbuf, "fmt ", 4))
	goto fail;

    /* format length */
    hdr->formatlen = FileReadL(fp, swap);
    hdr->wFormatTag = FileReadS(fp, swap);

    /* make sure format is understood */
    if (!verify_format(hdr))
	goto fail;

    hdr->wChannels = FileReadS(fp, swap);
    hdr->dwSamplesPerSec = FileReadL(fp, swap);
    hdr->dwAvgBytesPerSec = FileReadL(fp, swap);
    hdr->wBlockAlign = FileReadS(fp, swap);

    hdr->datasize = FileReadS(fp, swap);
    switch (hdr->datasize) {
    case 8:
    case 16:
    case 32:
	break;
    default:
	fprintf(stderr, "Bogus data size %d\n", hdr->datasize);
	goto fail;
    }

    if (!fread(tbuf, 4, 1, fp))
	goto fail;
    if (strncmp(tbuf, "data", 4))
	goto fail;

    hdr->datalen = FileReadL(fp, swap);
    hdr->data_start_offset = ftell(fp);

    if (comment) {
	*comment = FileCommentFromFilename(fname);
    }
    return fp;

fail:
    if (fp && strcmp(fname, "-"))
	fclose(fp);
    return (FILE *) 0;
}

/* XXX do something with comment */
/* ARGSUSED */
FILE       *
WaveOpenFileForWriting(fname, hdr, comment)
    _WaveConst char *fname;
    WaveHeader *hdr;
    char       *comment;
{
    FILE       *fp;
    int         swap;
    unsigned long tlen;
    int         endian = 1;

    if (strcmp(fname, "-") == 0) {
	fp = stdout;
    } else {
	fp = fopen(fname, "w");
	if (!fp)
	    return fp;
    }
    if (*(char *) &endian)
	swap = 0;
    else
	swap = 1;
    tlen = hdr->datasize / 8 * hdr->wChannels * hdr->datalen;
    fputs("RIFF", fp);
    FileWriteL(tlen, fp, swap);
    fputs("WAVE", fp);
    fputs("fmt ", fp);

    FileWriteL(hdr->formatlen, fp, swap);
    FileWriteS(hdr->wFormatTag, fp, swap);
    FileWriteS(hdr->wChannels, fp, swap);
    FileWriteL(hdr->dwSamplesPerSec, fp, swap);
    FileWriteL(hdr->dwAvgBytesPerSec, fp, swap);
    FileWriteS(hdr->wBlockAlign, fp, swap);
    FileWriteS(hdr->datasize, fp, swap);
    fputs("data", fp);
    FileWriteL(hdr->datasize, fp, swap);

    return fp;
}
