/*
 * Copyright 1993 Network Computing Devices, Inc.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name Network Computing Devices, Inc. not be
 * used in advertising or publicity pertaining to distribution of this
 * software without specific, written prior permission.
 * 
 * THIS SOFTWARE IS PROVIDED 'AS-IS'.  NETWORK COMPUTING DEVICES, INC.,
 * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT
 * LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NONINFRINGEMENT.  IN NO EVENT SHALL NETWORK
 * COMPUTING DEVICES, INC., BE LIABLE FOR ANY DAMAGES WHATSOEVER, INCLUDING
 * SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING LOSS OF USE, DATA,
 * OR PROFITS, EVEN IF ADVISED OF THE POSSIBILITY THEREOF, AND REGARDLESS OF
 * WHETHER IN AN ACTION IN CONTRACT, TORT OR NEGLIGENCE, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * $NCDId: @(#)wave.h,v 1.6 1993/02/26 23:56:02 lemke Exp $
 */

/*
 * WAVE support library
 */

#ifndef _WAVE_H_
#define _WAVE_H_

/*
 * If we are being used outside the NCD-AUDIO sound library, then we'll need
 * some ANSIfication definitions.
 */

#ifndef NeedFunctionPrototypes

#if defined(FUNCPROTO) || __STDC__ || defined(__cplusplus) || defined(c_plusplus)
#define NeedFunctionPrototypes 1
#else
#define NeedFunctionPrototypes 0
#endif

#endif				/* NeedFunctionPrototypes */

#ifndef _WaveConst

#if __STDC__ || defined(__cplusplus) || defined(c_plusplus) || (FUNCPROTO&4)
#define _WaveConst const
#else
#define _WaveConst		/**/
#endif

#endif				/* _WaveConst */

typedef struct _WaveHeader {
/* read from file */
    unsigned short wFormatTag;
    unsigned short wChannels;
    unsigned long dwSamplesPerSec;
    unsigned long dwAvgBytesPerSec;
    unsigned short wBlockAlign;
/* additional/extrapolated stuff */
    char        swap;
    unsigned long datalen;
    unsigned short datasize;
    unsigned short formatlen;
    unsigned short data_start_offset;
}           WaveHeader;

/* WAVE file format */
/* XXX Microsoft doc is crap -- this comes from a combination of it
 * and Rick Richardson's work for SOX
 */

/*
'RIFF'
ulong datasize
'WAVE'
'fmt '
ushort format chunk size
WaveHeader
    ushort format type
    ushort channels
    ulong samples per second
    ulong avg bytes per second
    ushort block align (channels * samplesize + 7) / 8
    ushort size of sound data (8, 16, etc)
'data'
    sound data
    ulong bytes of sound data
*/


/* format types */
#define	WAVE_FORMAT_PCM		0x0001	/* Microsoft PCM */
#define	IBM_FORMAT_MULAW	0x0101	/* IBM mu-law format */
#define	IBM_FORMAT_ALAW		0x0102	/* IBM a-law format */
#define	IBM_FORMAT_ADPCM	0x0103	/* IBM AVC Adaptive Differential Pulse
					 * Code Modulation format */

#define	WAVE_MAGIC		0x57415645	/* WAVE in hex */

extern FILE *
WaveOpenFileForReading(
#if NeedFunctionPrototypes
		       _WaveConst char *,	/* file name */
		       WaveHeader *,		/* fillin_header */
		       char **			/* ret_comment */
#endif
);

extern FILE *
WaveOpenFileForWriting(
#if NeedFunctionPrototypes
		       _WaveConst char *,	/* file name */
		       WaveHeader *,		/* fillin_header */
		       char *			/* comment */
#endif
);

#ifndef mips
extern int
WaveFormatLength(
#if NeedFunctionPrototypes
    			unsigned short		/* format */
#endif
);
#endif

#endif				/* _WAVE_H_ */
